# /www/psets/1-quotations/insert-quote-2.tcl
#
# Copyright (C) 1998-2007 various parties
#
# This program is free software. You can redistribute it and/or modify it under 
# the terms of the GNU General Public License as published by the Free Software 
# Foundation; either version 2 of the License, or (at your option) any later version. 
# This program is distributed in the hope that it will be useful, but WITHOUT ANY 
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# Edited by frank.bergmann@project-open.com

# Page Contract: Check the parameter for validity and provide comments and
# parameter descriptions for the OpenACS internal documentation system.
# Nice!
ad_page_contract {
    Takes the quotation values from the quotation form
    and stores them into the database.
    @author frank.bergmann@project-open.com

    @param quotation_id The ID for the new object from the last page
           Setting this ID in the last page means that the object
           won't be created twice if the user should double-click
           the Submit button...
    @param quotation_text
    @param return_url Where to go after we finisched the operation?
           Defaults to the "index" page of the local directory.
} {
    quotation_id:integer
    quotation_text
    { return_url "index" }
}

# Find out who is currently logged on.
# This commands redirects to a login screen (and returns to this page!) 
# if the user hasn't been logged on yet.
set user_id [ad_maybe_redirect_for_registration]


# The SQL to insert the quotation into the DB.
# Please observe the ":" colon variables.
# The values for these variables are passed to the database
# driver separately from the SQL, so that there is no chance
# for "SQL Injection" attacks here.
#
set insert_sql "
	insert into quotations (
		quotation_id,
		quotation_user_id,
		quotation_text
	) values (
		:quotation_id,
		:user_id,
		:quotation_text
	)
"


# Execute the SQL and catch the error
set error_p [catch {
  db_dml insert_quotation $insert_sql
} err_msg]

# Return an error message if there was an error.
if {$error_p} {
    ad_return_complaint 1 "
	<b>Error inserting quote into the database</b><br>
	Either the database table doesn't exist, or you are<br>
 	trying to insert the same quotation twice...<br>
	The error message below will help you to diagnose:<br>
	<pre>$err_msg</pre>
    "
    ad_script_abort
}


# Returns to the previous screen. Nice, isn't it?
ad_returnredirect $return_url
