
<html>
<head>
<title>File Operations</title>
<LINK REL=STYLESHEET HREF="tcl_manual.css" TYPE="text/css">
</head>
<body bgcolor=white text=black>

<h2>File Operations</h2>

part of <a href="index.adp">Tcl for Web Nerds</a> 
by 
<a href="http://www-swiss.ai.mit.edu/~hal/hal.html">Hal Abelson</a>,
<a href="http://philip.greenspun.com/">Philip Greenspun</a>,
and
<a href="http://sandon.org/l/">Lydia Sandon</a>


<hr>


Tcl has a built-in interface for dealing with Unix files.  The
commands themselves are relatively straightforward, so we'll just
explain them in a reference list below.

<H3>Reference</H3>

<UL>

<LI><code>file atime <i>filename</i></code><br>
<i>Returns as a decimal number the time that the file was last accessed.</i>

<blockquote>
<code><pre>
set access_time [file atime "index.adp"] <i>==> 916612934 </i>
</pre></code>
</blockquote>


<LI><code>file dirname <i>filename</i></code><br>
<i>Returns the name of the parent directory of the file.</i>
<blockquote>
<code><pre>
set parent_dir [file dirname "~/home/dir/this.adp"] <i>==> ~/home/dir</i>
</pre></code>
</blockquote>


<LI><code>file executable <i>filename</i></code><br>
<i>Returns 1 if the file is executable, 0 otherwise.</i>
<blockquote>
<code><pre>
chmod 1111 billg-wealth.tcl
file executable billg-wealth.tcl <i>==> 1</i>
</pre></code>
</blockquote>


<LI><code>file exists <i>filename</i></code> <br>
<i>Returns 1 if the file exists, 0 otherwise.</i>

<blockquote>
<code><pre>
file exists billg-wealth.tc <I>==> 0</i>
file exists billg-wealth.tcl <I>==> 1</i>
</pre></code>
</blockquote>


<LI><code>file extension <i>filename</i></code><br>
<i>Returns the file extension of the file (i.e. from the last dot to
the end)</i>

<blockquote>
<code><pre>
file extension billg-wealth.tcl <i>==> .tcl</i>
</pre></code>
</blockquote>


<LI><code>file isdirectory <i>filename</i></code><br>
<i>Returns 1 if the file is a directory, 0 otherwise.</i>

<blockquote>
<code><pre>
file isdirectory . <i>==> 1</i>
file isdirectory billg-wealth.tcl <i>==> 0</i>
</pre></code>
</blockquote>


<LI><code>file isfile <i>filename</i></code><br>
<i>Returns 1 if the file is not a directory, symbolic link, or
device, 0 otherwise.</i>

<blockquote>
<code><pre>
file isfile billg-wealth.tcl <i>==> 1</i>
</pre></code>
</blockquote>


<LI><code>file lstat <i>filename variablename</i></code><br>
<i>Puts the results of the <code>stat</code> command on linkname into
variablename.</i>

<blockquote>
<code><pre>
<i>ln -s billg-wealth.tcl temp.tcl</i>
file lstat temp.tcl temp <i>==> (array holding stat info)</i>
</pre></code>
</blockquote>


<LI><code>file mtime <i>filename</i> </code><br>
<i>Returns the modify time of file as a decimal string.</i>

<blockquote>
<code><pre>
file modify billg-wealth.tcl <i>==> 915744902</i>
</pre></code>
</blockquote>


<LI><code>file owned <i>filename</i></code><br>
<i>Returns 1 if the current user owns the file, else 0.</i>

<blockquote>
<code><pre>
file owned billg-wealth.tcl <i>==> 1</i>
</pre></code>
</blockquote>


<LI><code>file readable <i>filename</i></code><br>
<i>Returns 1 if the file is readable, else 0.</i>

<blockquote>
<code><pre>
file readable billg-wealth.tcl <i>==> 1</i>
</pre></code>
</blockquote>


<LI><code>file readlink <i>filename</i></code><br>
<i>Returns the contents of the symbolic link named filename.</i>

<blockquote>
<code><pre>
<i>ln -s file.txt file1.txt</i>
file readlink file1.txt <i>==> file.txt</i>
</pre></code>
</blockquote>


<LI><code>file rootname <i>filename</i></code><br>
<i>Returns all but the extension and the last . of the filename.</i>

<blockquote>
<code><pre>
file rootname billg-wealth.tcl <i>==> billg-wealth</i>
</pre></code>
</blockquote>


<LI><code>file size <i>filename</i></code><br>
<i>Returns the size in bytes of the file.</i>

<blockquote>
<code><pre>
file size billg-wealth.tcl <i>==> 774</i>
</pre></code>
</blockquote>


<LI><code>file stat <i>filename variablename</i></code><br>
<i>Returns the stat results about the file into the array named
variablename.  The elements of the variable array are: atime, ctime,
dev, gid, ino, mode, mtime, nlink, size, type, and uid.</i>

<blockquote>
<code><pre>
file stat billg-wealth.tcl billg_info 
set $billg_info(ctime) <i>==> 916615489</i>
</pre></code>
</blockquote>


<LI><code>file tail <i>filename</i></code><br>
<i>Returns all of the characters after the last / in the filename.</i>

<blockquote>
<code><pre>
file tail ~/home/dir/subdir/file.txt <i>==> file.txt</i>
</pre></code>
</blockquote>


<LI><code>file type <i>filename</i></code><br>
<i>Returns the type identified of the filename arg, which can be one of the
following: file, directory, characterSpecial, blockSpecial, fifo,
link, or socket.</i>

<blockquote>
<code><pre>
file type billg-wealth.tcl <i>==> file</i>
</pre></code>
</blockquote>


<LI><code>file writable <i>filename</i></code><br>
<i>Returns 1 if the file is writable, 0 otherwise.</i>

<blockquote>
<code><pre>
file writable billg-wealth.tcl <i>==> 0</i>
</pre></code>
</blockquote>

</UL>


More:  <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/file.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/file.n.html</a>

<H3>Input/Output Commands</H3>

<UL>

<LI><code>open <i>filename ?access? ?permissions?</i></code><br>
<i>Returns a stream handle to open the file for the access specified
with the permissions specified. Defaut values are read for the access
required and the permissions are the same as the default permissions
on a file.  The access value options are r (read from existing file), r+
(read from and write to existing file), w (write over or create and
write to file as necessary), w+ (read from and write to or create file
as necessary), a (write to existing file; append data to it), a+ (read
from and write to existing file; append data to it).</i>
<blockquote>
<code><pre>
set this_file_stream [open /tmp/file.txt r]
</pre></code>
</blockquote>


More: <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/open.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/open.n.html</a>

<br>
<br>

<LI><code>puts <i>?-nonewline? ?stream? string</i></code><br>
<i>Write the string to the stream. Default is STDOUT.</i>

<blockquote>
<code><pre>
puts "Hello, world." <i>==> Hello, world.</i>
</pre></code>
</blockquote>


More: <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/puts.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/puts.n.html</a>

<br>
<br>

<LI><code>gets <i>stream ?varname?</i></code><br>
<i>Read a line from the stream. If a variable is specified, put the
line into that variable.</i>

<blockquote>
<code><pre>
gets $thisstream line
</pre></code>
</blockquote>


More: <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/gets.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/gets.n.html</a>

<br>
<br>

<LI><code>read <i>streamname ?numbytes?</i></code><br>
<i>If numbytes is specified, read that many bytes of the stream. If
not, read the whole stream.</i>

<blockquote>
<code><pre>
set first_ten_bytes [read $this_stream 10]
</pre></code>
</blockquote>


More: <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/read.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/read.n.html</a>

<br>
<br>

<LI><code>read -nonewline <i>streamname</i></code><br>
<i>Read the whole stream and discard the last newline.</i>

<blockquote>
<code><pre>
set this_file_contents [read -nonewline $this_stream]
</pre></code>
</blockquote>


<LI><code>tell <i>streamname</i></code><br>
<i>Return the "seek offset." (See below for seek.)</i>

<blockquote>
<code><pre>
set seek_offset [tell $this_stream]
</pre></code>
</blockquote>


More: <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/tell.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/tell.n.html</a>

<br>
<br>

<li><code>seek <i>streamname offset ?origin?</i></code><br>
<i>Set the seek offset. Origin is either <code>start, current, or
end</code>.</i>

<blockquote>
<code><pre>
seek $this_stream offset end
</pre></code>
</blockquote>


More: <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/seek.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/seek.n.html</a>

<br>
<br>

<LI><code>eof <i>stream</i></code><br>
<i>Returns 1 if you have reached the end of the stream; 0 otherwise.</i>

<blockquote>
<code><pre>
if {[eof $this_stream]} {
   break
}
</pre></code>
</blockquote>


More: <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/eof.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/eof.n.html</a>

<br>
<br>

<LI><code>flush <i>streamname</i></code><br>
<i>Write buffers of a stream</i>

<blockquote>
<code><pre>
flush $this_stream
</pre></code>
</blockquote>


More: <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/flush.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/flush.n.html</a>

<br>
<br>

<li><code>close <i>streamname</i></code><br>
<i>Close the stream.</i>

<blockquote>
<code><pre>
close $this_stream
</pre></code>
</blockquote>


More: <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/close.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/close.n.html</a>

</UL>


<br>
<br>

Continue on to <A HREF="eval.adp"><code>eval</code></A>.



<hr>
Return to <a href="index.adp">Table of Contents</a>
<br>
<br>
<address><a href="mailto:lsandon@alum.mit.edu">lsandon@alum.mit.edu</a></address>

<center><a href="/comments/add?page_id=4735">Add a comment</a> | <a href="/links/add?page_id=4735">Add a link</a></center>

</body>
</html>

