
<html>
<head>
<title>Pattern matching</title>
<LINK REL=STYLESHEET HREF="tcl_manual.css" TYPE="text/css">
</head>
<body bgcolor=white text=black>

<h2>Pattern matching</h2>

part of <a href="index.adp">Tcl for Web Nerds</a> 
by 
<a href="http://www-swiss.ai.mit.edu/~hal/hal.html">Hal Abelson</a>,
<a href="http://philip.greenspun.com/">Philip Greenspun</a>,
and
<a href="http://sandon.org/l/">Lydia Sandon</a>


<hr>


Pattern matching is important across a wide variety of Web programming
tasks but most notably when looking for exceptions in user-entered
data and when trying to parse information out of non-cooperating Web
sites.

<p>

Tcl's pattern matching facilities test whether a given string matches
a specified <em>pattern</em>.  Patterns are described using a syntax
known as <em>regular expressions</em>.  For example, the pattern
expression consisting of a single period matches any character.  The
pattern <code>a..a</code> matches any four-character string whose
first and last characters are both <code>a</code>.

<p>
The <code>regexp</code> command takes a pattern, a string, and an
optional match variable. It tests whether the string matches the
pattern, returns 1 if there is a match and zero otherwise, and sets the
match variable to the part of the string that matched the
pattern:

<blockquote>
<code><pre>
% set something candelabra
candelabra
% regexp a..a $something match
1
% set match
abra
</pre></code>
</blockquote>


Patterns can also contain subpatterns (delimited by
parentheses) and denote repetition.  A star denotes zero or more
occurrences of a pattern, so <code>a(.*)a</code> matches any string of
at least two characters
that begins and ends with the character <code>a</code>.  Whatever has
matched the subpattern between the a's will get put into the first
subvariable:

<blockquote>
<code><pre>
% set something candelabra
candelabra
% regexp a(.*)a $something match
1
% set match
andelabra
</pre></code>
</blockquote>


Note that Tcl regexp by default behaves in a <em>greedy</em> fashion.
There are three alternative substrings of "candelabra" that match the
regexp <code>a(.*)a</code>:  "andelabra", "andela", and "abra".  
Tcl chose the
longest substring.  This is very painful when trying to pull HTML 
pages apart:

<blockquote>
<code><pre>
% set simple_case "Normal folks might say &lt;i&gt;et cetera&lt;/i&gt;"
Normal folks might say &lt;i&gt;et cetera&lt;/i&gt;
% regexp {&lt;i&gt;(.+)&lt;/i&gt;} $simple_case match italicized_phrase
1
% set italicized_phrase
et cetera
% set some_html "Pedants say &lt;i&gt;sui generis&lt;/i&gt; and &lt;i&gt;ipso facto&lt;/i&gt;"
Pedants say &lt;i&gt;sui generis&lt;/i&gt; and &lt;i&gt;ipso facto&lt;/i&gt;
% regexp {&lt;i&gt;(.+)&lt;/i&gt;} $some_html match italicized_phrase
1
% set italicized_phrase
sui generis&lt;/i&gt; and &lt;i&gt;ipso facto
</pre></code>
</blockquote>


What you want is a non-greedy regexp, a standard feature of Perl and
an option in Tcl 8.1 and later versions (see <a
href="http://www.scriptics.com/services/support/howto/regexp81.html">http://www.scriptics.com/services/support/howto/regexp81.html</a>).

<p class="marginnote">

Lisp systems in the 1970s included elegant ways of returning all
possibilities when there were multiple matches for an expression.
Java libraries, Perl, and Tcl demonstrate the progress of the field of
computer science by ignoring these superior systems of decades past.

<h3>Matching Cookies From the Browser</h3>

A common problem in Web development is pulling information out of
cookies that come from the client.  The cookie spec 
at <a
href="http://home.netscape.com/newsref/std/cookie_spec.html">http://home.netscape.com/newsref/std/cookie_spec.html</a>
mandates that multiple cookies be separated by semicolons.  So you
look for "the cookie name that you've been using" followed by an
equals sign and them slurp up anything that follows that isn't a
semicolon.  Here is how the ArsDigita Community System looks for the
value of the last_visit cookie:

<blockquote>
<code><pre>
regexp {last_visit=([^;]+)} $cookie match last_visit
</pre></code>
</blockquote>


Note the square brackets inside the regexp.  The Tcl interpreter isn't
trying to call a procedure because the entire regexp has been grouped
with braces rather than double quotes.  Square brackets denote a range
of acceptable characters:
<ul>
<li><code>[A-Z]</code> would match any uppercase character

<li><code>[ABC]</code> would match any of first three characters in
the alphabet (uppercase only)

<li><code>[^ABC]</code> would match any character <em>other</em> than
the first three uppercase characters in the alphabet, i.e., the
<code>^</code> reverses the sense of the brackets
</ul>

The plus sign after the <code>[^;]</code> says "one or more characters
that meets the preceding spec", i.e., "one or more characters that
isn't a semicolon".  It is distinguished from <code>*</code> in that
there must be at least one character for a match.

<p>

If successful, the <code>regexp</code> command above will set the
<code>match</code> variable with the complete matching string,
starting from "last_visit=".  Our code doesn't make any use of this
variable but only looks at the subvar <code>last_visit</code> that
would also have been set.

<p>

Pages that use this cookie expect an integer and this code failed in
one case where a user edited his cookies file and corrupted it so that
his browser was sending several thousands bytes of garbage after the
"last_visit=".  A better approach might have been to limit the match
to digits:

<blockquote>
<code><pre>
regexp {last_visit=([0-9]+)} $cookie match last_visit
</pre></code>
</blockquote>


<h3>Matching Into Multiple Variables</h3>

More generally <code>regexp</code> allows multiple pattern variables.
The pattern variables after the first are set to the substrings that
matched the subpatterns.  Here is an example of matching a credit card
expiration date entered by a user:

<blockquote>
<code><pre>
% set date_typed_by_user "06/02"
06/02
% regexp {([0-9][0-9])/([0-9][0-9])} $date_typed_by_user match month year
1
% set month
06
% set year
02
% 
</pre></code>
</blockquote>


Each pair of parentheses corresponds to a subpattern variable.

<h3>Full Syntax</h3>

The most general form of <code>regexp</code> includes
optional flags as well as multiple match variables:

<blockquote>
<code><pre>
regexp <I>[flags] pattern data matched_result var1 var2 ...</I>
</pre></code>
</blockquote>


The various flags are

 <UL>
 <LI><CODE>-nocase</CODE><BR>
 <I>uppercase characters in the data are bashed down to lower 
for case-insensitive matching
(make sure that your pattern is all lowercase!)</I>
 <LI><CODE>-indices</CODE><BR>
 <I>the returned values of the regexp contain the indices delimiting the
 matched substring, rather than the strings themselves.</I>
 <LI><I>If your pattern begins with a <CODE>-</CODE>, put a
 <CODE>--</CODE> flag at the end of your flags</I>
 </UL>

Regular expression syntax is:

 <UL>
 <LI><CODE>.</CODE><BR><I>matches any character.</I></LI>
 <LI><CODE>*</CODE><BR><I>matches zero or more instances of the
 previous pattern item.</I></LI>
 <LI><CODE>+</CODE><BR><I>matches one or more instances of the previous
 pattern item.</I></LI>
 <LI><CODE>?</CODE><BR><I>matches zero or one instances of the
 previous pattern item.</I></LI>
 <LI><CODE>|</CODE><BR><I>disjunction, e.g., <code>(a|b)</code> matches
 an <code>a</code> or a <code>b</code></i></LI>
 <LI><CODE>( )</CODE><BR><I>groups a sub-pattern.</I></LI>
 <LI><CODE>[ ]</CODE><BR><I>delimits a set of characters. ASCII Ranges
 are specified using hyphens, e.g., <code>[A-z]</code> matches any
 character from uppercase <code>A</code> through lowercase
 <code>z</code> (i.e., any alphabetic character).  If the first
 character in the set is <code>^</code>, this
 complements the set, e.g., <code>[^A-z]</code>
 matches any non-alphabetic character.</I></LI>

  <LI><CODE>^</CODE><BR><I>Matches only when the pattern
  appears at the beginning of the
  string.
  The <CODE>^</CODE> must appear at the beginning of the pattern expression.</I></LI>

 <LI><CODE>$</CODE><BR><I>Matches only when the pattern 
  appears at the end of the
  string. The <CODE>$</CODE> must appear last in the pattern expression.</I></LI>
 </UL>

Also see <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/regexp.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/regexp.n.html</a>


<h3>Matching with substitution</h3>

It's common in Web programming to create strings by substitution.
Tcl's <code>regsub</code> command performs substitution based on a
pattern:

<blockquote>
<code><pre>
regsub <i>[flags] pattern data replacements var</i>
</pre></code>
</blockquote>


matches the pattern against the data.  If the match succeeds, the
variable named <code>var</code> is set to <code>data</code>, with
various parts modified, as specified by <code>replacements</code>.  If
the match fails, <code>var</code> is simply set to
<code>data</code>.  The value returned by <code>regsub</code> is the
number of replacements performed.

<p>

The flag <code>-all</code> specifies that every occurrence of the
pattern should be replaced.  Otherwise only the first occurrence is
replaced.  Other flags include <code>-nocase</code> and <code>--</code>
as with <code>regexp</code>

<p class="stb">

Here's an example from the banner ideas module of the ArsDigita
Community System (see <a
href="http://photo.net/doc/bannerideas.html">http://photo.net/doc/bannerideas.html</a>).
The goal is that each banner idea contain a linked thumbnail image.
To facilitate cutting and pasting of the image html, we don't require
that the publisher include uniform subtags within the IMG.  However,
we use <code>regexp</code> to clean up:

<blockquote>
<code><pre>
# turn "&lt;img align=right hspace=5" into "&lt;img align=left border=0 hspace=8"
regsub -nocase {align=[^ ]+} $picture_html "" without_align
regsub -nocase {hspace=[^ ]+} $without_align "" without_hspace
regsub -nocase {&lt;img} $without_hspace {&lt;img align=left border=0 hspace=8} final_photo_html
</pre></code>
</blockquote>


<p class="stb">


In the example above, &lt;replacements&gt; specified the literal characters
<code>''</code>.  Other replacement directives include:

<ul>
<li><code>&amp;</code> inserts the string that matched the pattern
<li> The backslashed numbers <code>\1</code> through <code>\9</code>
inserts the strings that matched the corresponding sub-patterns in the
pattern.
</ul>

Here's another web example, which parses HTML, and replaces the
comments (delineated in HTML by <code>&lt;!--</code> and
<code>--&gt;</code>) by the comment text, enclosed in parentheses.

<blockquote>
<code><pre>
% proc extract_comment_text {html} {
    regsub -all {&lt;!--([^-]*)--&gt;} $html {(\1)} with_exposed_comments
    return $with_exposed_comments
}

% extract_comment_text {&lt;!--insert the price below--&gt;
We give the same low price to everyone: $219.99
&lt;!--make sure to query out discount if this is one of our big customers--&gt;}
(insert the price below)
We give the same low price to everyone: $219.99
(make sure to query out discount if this is one of our big customers)
</pre></code>
</blockquote>


Also see <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/regsub.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/regsub.n.html</a>

<h3>String match</h3>

Tcl provides an alternative matching mechanism that is simpler for
users to understand than regular expressions.  The Tcl command
<code>string match</code> uses "GLOB-style" matching.  Here is the
syntax:

<blockquote>
<code><pre>
string match <i>pattern data</i>
</pre></code>
</blockquote>


It returns 1 if there is
a match and 0 otherwise.  The only pattern elements permitted here are
<code>?</code>, which matches any single character; <code>*</code>,
which matches any sequence; and <code>[]</code>, which delimits a set
of characters or a range.  This differs from <code>regexp</code> in
that the pattern must match the entire string supplied:

<blockquote>
<code><pre>
% regexp "foo" "foobar"
1
% string match "foo" "foobar"
0
% # here's what we need to do to make the string match 
% # work like the regexp
% string match "*foo*" foobar
1
</pre></code>
</blockquote>


Here's an example of the character range system in use:

<blockquote>
<code><pre>
string match {*[0-9]*} $text
</pre></code>
</blockquote>


returns 1 if <code>text</code> contains at least one digit and 0
otherwise.

<br>
<br>

Also see <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/string.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/string.n.html</a>


<br>
<br>

Continue on to <A HREF="array-operations.adp">Array Operations</A>.


<hr>
Return to <a href="index.adp">Table of Contents</a>
<br>
<br>
<address><a href="mailto:lsandon@alum.mit.edu">lsandon@alum.mit.edu</a></address>

<center><h3>Reader's Comments</h3></center>
	<blockquote>
		<p>[A-z] includes characters other than A-Z and a-z (see the ASCII man page).  I ran a test using tclsh 8.3:
<br>
<pre>
% set x ^
^
% regsub {[A-z]} $x {OOPS} x2
1
% puts $x2
OOPS
</pre>

		<br><br>

-- <A HREF="/shared/community-member?user_id=261985">Paul Takemura</a>, January 29, 2005</blockquote>

	<center>
	<a href="/comments/add?page_id=4734">Add a comment</a>
	</center>
	<center><h3>Related Links</h3></center>
	<ul><li><a href="http://www.tcl.tk/doc/howto/regexp81.tml#part2_1">nongreedy regular expressions</a>- nongreedy regular expressions changed the url to this new one&nbsp;&nbsp;  <font size=-1>(contributed by <A HREF="/shared/community-member?user_id=264446">Ralph Bremer</a>)</font>
<p>
</ul>
	<center>
	<a href="/links/add?page_id=4734">Add a link</a>
	</center>

</body>
</html>


