
<html>
<head>
<title>Procedures</title>
<LINK REL=STYLESHEET HREF="tcl_manual.css" TYPE="text/css">
</head>
<body bgcolor=white text=black>

<h2>Procedures</h2>

part of <a href="index.adp">Tcl for Web Nerds</a> 
by 
<a href="http://www-swiss.ai.mit.edu/~hal/hal.html">Hal Abelson</a>,
<a href="http://philip.greenspun.com/">Philip Greenspun</a>,
and
<a href="http://sandon.org/l/">Lydia Sandon</a>


<hr>


To define a procedures in Tcl use the following syntax:

<blockquote>
<code><pre>
proc <I>name</I> { <I>list_of_arguments</I> } {
   <I>body_expressions</I>
}
</pre></code>
</blockquote>


This creates a procedure with the name "name."  Tcl has a global
environment for procedure names, i.e., there can be only one procedure
called "foobar" in a Tcl system.

<P>

The next part of the syntax is the set of arguments, delimited by a
set of curly braces.  Each argument value is then mapped into the
procedure body, which is also delimited by curly braces.  As before,
each statement of the procedure body can be separated by a semi-colon
or a newline (or both).  Here's an example, taken from 
<a href="http://photo.net/doc/calendar-widget.html">the calendar widget</a> 
component of the ArsDigita Community System:


<blockquote>
<code><pre>
proc calendar_convert_julian_to_ansi { date } {
    set db [ns_db gethandle subquery]
    # make Oracle do all the real work
    set output [database_to_tcl_string $db \ 
        "select trunc(to_date('$date', 'J')) from dual"]
    ns_db releasehandle $db
    return $output
}
</pre></code>
</blockquote>


Of course in the actual ACS, we encourage programmers to use
<code>proc_doc</code> instead, which defines the procedure but also
records a doc string, as described in <a
href="array-operations.adp">array variable chapter</a>.  Here's what
the definition would look like with <code>proc_doc</code>:

<blockquote>
<code><pre>
proc_doc calendar_convert_julian_to_ansi { date } "Return an ANSI date
for a Julian date" {
    set db [ns_db gethandle subquery]
    # make Oracle do all the real work
    set output [database_to_tcl_string $db \ 
        "select trunc(to_date('$date', 'J')) from dual"]
    ns_db releasehandle $db
    return $output
}
</pre></code>
</blockquote>


<br>
<br>

More:  <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/proc.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/proc.n.html</a>

<H3>Scope, Upvar and Uplevel</H3>

There are three possible scopes for a variable in an AOLserver Tcl
script:

<ul>
<li>local to a procedure (the default)
<li>shared among all procedures executing in one thread
(<code>global</code>)
<li>shared among all threads on a server and persistent from one
connection to another (<code>ns_share</code>)
</ul>

<P>

To use a variable locally, you need not declare it.  To instruct the
Tcl interpreter to read and set a variable in the global environment,
you must call <code>global</code> every place that the variable is
used.  For example, when side graphics have been displayed on a page,
<code>ad_footer</code> needs to know so that it can insert a <code>&lt;BR
CLEAR=RIGHT&gt;</code> tag.  

<blockquote>
<code><pre>
# a proc that might display a side graphic
proc ad_decorate_side {} {
    # we use a GLOBAL variable (shared by procs in a thread) as opposed to 
    # an ns_share (shared by many threads)
    <font color=red>global sidegraphic_displayed_p</font>
    ...
    set sidegraphic_displayed_p 1
}

proc ad_footer {{signatory ""}} {
    <font color=red>global sidegraphic_displayed_p</font>
    if [empty_string_p $signatory] {
	set signatory [ad_system_owner]
    } 
    if { [info exists sidegraphic_displayed_p] && $sidegraphic_displayed_p } {
	# we put in a BR CLEAR=RIGHT so that the signature will clear any side graphic
	# from the ad-sidegraphic.tcl package
	set extra_br "&lt;br clear=right&gt;"
    } else {
	set extra_br ""
    }
    return "
$extra_br
&lt;hr&gt;
&lt;a href=\"mailto:$signatory\"&gt;&lt;address&gt;$signatory&lt;/address&gt;&lt;/a&gt;
&lt;/body&gt;
&lt;/html&gt;"
}
</pre></code>
</blockquote>


One of the strangest and most difficult to use features of Tcl is the
ability to read and write variables up the calling stack with 
<code>uplevel</code> and <code>upvar</code>.

<br>
<br>

More:  <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/upvar.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/upvar.n.html</a>; <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/uplevel.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/uplevel.n.html</a>; <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/global.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/global.n.html</a>


<H3>Optional arguments</H3>

Here is an example of a procedure that has one required and one
optional argument:

<blockquote>
<code><pre>
proc ad_header {page_title {extra_stuff_for_document_head ""}} {
    set html "&lt;html&gt;
&lt;head&gt;$extra_stuff_for_document_head
&lt;title&gt;$page_title&lt;/title&gt;
&lt;/head&gt;
"
    return $html
}
</pre></code>
</blockquote>


If a page calls <code>ad_header</code> with one argument, it gets the
standard appropriate HTML header.  If a page supplies the extra
optional argument, that information gets written into the HEAD.
Otherwise, the default value of empty string is used for the second
argument.


<H3>Variable number of arguments</H3>

In addition, Tcl can also provide for a variable number of arguments
at the end, using a special last argument called <code>args</code> in
any procedure definition.  After all of the other (previous) arguments
are bound to names, the rest of the arguments are shoved into a list
called <code>args</code> which can then be accessed inside the procedure.

<P>

You could imagine that between optional arguments and extra ones, the
interpreter might get confused.  It doesn't, because it assumes that
you aren't using extra args at the end without binding all of the
optional ones in the middle; that is, it stuffs argument values into
the argument names in strict order without regard to options, extras,
etc.

<H3>Rename</H3>

In case this wasn't flexible enough, Tcl lets you rename procedures
using a proc called <code>rename <i>old_name new_name</I></code>.  If
you wanted to keep the old proc but still take advantage of the great
abstraction you've used throughout your site (i.e. not change all of
the command calls when you change their bodies), you can rename the
old proc and create a new one with the old one's name.  You can 
also use <code>rename</code> with the empty string for the second
argument.  This results in the proc being deleted.

<br>
<br>

More:  <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/rename.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/rename.n.html</a>

<br>
<br>

Continue on to <A HREF="file-commands.adp">file commands</A>.


<hr>
Return to <a href="index.adp">Table of Contents</a>
<br>
<br>
<address><a href="mailto:lsandon@alum.mit.edu">lsandon@alum.mit.edu</a></address>

<center><a href="/comments/add?page_id=4733">Add a comment</a> | <a href="/links/add?page_id=4733">Add a link</a></center>

</body>
</html>

