
<html>
<head>
<title>Basic String Operations</title>
<LINK REL=STYLESHEET HREF="tcl_manual.css" TYPE="text/css">
</head>
<body bgcolor=white text=black>

<h2>Basic String Operations</h2>

part of <a href="index.adp">Tcl for Web Nerds</a> 
by 
<a href="http://www-swiss.ai.mit.edu/~hal/hal.html">Hal Abelson</a>,
<a href="http://philip.greenspun.com/">Philip Greenspun</a>,
and
<a href="http://sandon.org/l/">Lydia Sandon</a>


<hr>


If your program receives data from a Web client, it comes in as a
string.  If your program sends an HTML page back to a Web client, it
goes out as a string.  This puts the string data type at the heart of
Web page development:

<blockquote>
<code><pre>
set whole_page "some stuff for the top of the page\n\n"
append whole_page "some stuff for the middle of the page\n\n"
append whole_page "some stuff for the bottom of the page\n\n"
# done composing the page, let's write it back to the user
ns_return 200 text/html $whole_page
</pre></code>
</blockquote>


If you're processing data from the user, typically entered into an
HTML form, you'll be using a rich variety of built-in string-handling
procedures.  Suppose that a user is registering at your site with the
form variables <code>first_names, last_name, email, password</code>.
Here's how we might build up a list of exceptions (using the Tcl
<code>lappend</code> command, described in the chapter on lists):

<blockquote>
<code><pre>
# compare the first_names value to the empty string
if { [string compare $first_names ""] == 0 } {
    lappend exception_list "You forgot to type your first name"
}

# see if their email address has the form
#   something at-sign something
if { ![regexp {.+@.+} $email] } {
    lappend exception_list "Your email address doesn't look valid."
}

if { [string length $password] > 20 } {
    lappend exception_list "The password you selected is too long."
}
</pre></code>
</blockquote>


If there aren't any exceptions, we have to get these data ready for
insertion into the database:

<blockquote>
<code><pre>
# remove whitespace from ends of input (if any)
set last_name_trimmed [string trim $last_name]

# escape any single quotes with an extra one (since the SQL
# string literal quoting system uses single quotes)
regsub -all ' $last_name_trimmed '' last_name_final

set sql_insert "insert into users (..., last_name, ...) 
values 
(..., '$last_name_final', ...)"
</pre></code>
</blockquote>



<h3>Looking for stuff in a string</h3>

The simplest way to look for a substring within a string is with the
<code>string first</code> command.  Some users of photo.net complained
that they didn't like seeing classified ads that were simply pointers
to the eBay auction site.  Here's a simplified snippet from 
<a href="http://software.arsdigita.com/www/gc/place-ad-3.tcl">http://software.arsdigita.com/www/gc/place-ad-3.tcl</a>:

<blockquote>
<code><pre>
if { [string first "ebay" [string tolower $full_ad]] != -1 } {
    # return an exception
    ...
}
</pre></code>
</blockquote>


an alternative formulation would be 

<blockquote>
<code><pre>
if { [regexp -nocase {ebay} $full_ad] } {
    # return an exception
    ...
}
</pre></code>
</blockquote>


Both implementations will catch any capitalization variant of "eBAY".
Both implementations will miss "e-bay" but it doesn't matter because
if the poster of the ad includes a link with a URL, the hyperlink will
contain "ebay".  What about false positives?  If you visit <a
href="http://www.m-w.com">www.m-w.com</a> and search for "*ebay*"
you'll find that both implementations might bite someone selling
rhododendrons or a water-powered mill.  That's why the toolkit code
checks a "DisalloweBay" parameter, set by the publisher, before
declaring this an exception.

<p>

If you're just trying to find a substring, you can use either
<code>string first</code> or <code>regexp</code>.  If you're trying to
do something more subtle, you'll need regexp (described more fully in
the chapter "Pattern Matching"):

<blockquote>
<code><pre>
if { ![regexp {[a-z]} $full_ad] } {
    # no lowercase letters in the ad!
    append exception_text "<li>Your ad appears to be all uppercase.  
ON THE INTERNET THIS IS CONSIDERED SHOUTING.  IT IS ALSO MUCH 
HARDER TO READ THAN MIXED CASE TEXT.  So we don't allow it, 
out of decorum and consideration for people who may 
be visually impaired."
    incr exception_count
}
</pre></code>
</blockquote>


<h3>Using only part of a string</h3>

In the ArsDigita Community System, we have a page that shows a user's
complete history with a Web service, e.g., <a
href="http://photo.net/shared/community-member.tcl?user_id=23069">http://photo.net/shared/community-member.tcl?user_id=23069</a>
shows all of the postings by Philip Greenspun.  If a comment on a
static page is short, we want to show the entire message.  If not, we
want to show just the first 1000 characters.

<P>

In <a
href="http://software.arsdigita.com/www/shared/community-member.tcl">http://software.arsdigita.com/www/shared/community-member.tcl</a>,
we find the following use of the <code>string range</code> command:

<blockquote>
<code><pre>
if { [string length $message] > 1000 } {
    set complete_message "[string range $message 0 1000]... "
} else {
    set complete_message $message
}
</pre></code>
</blockquote>



<h3>Fortran-style formatting and reading of numbers</h3>

The Tcl commands <code>format</code> and <code>scan</code> resemble
C's <code>printf</code> and <code>scanf</code> commands.  That's
pretty much all that any Tcl manual will tell you about these
commands, which means that you're kind of S.O.L. if you don't know C.
The basic idea of these commands comes from Fortran, a computer
language developed by John Backus at IBM in 1954.  The FORMAT command
in Fortran would let you control the printed display of a number,
including such aspects as spaces of padding to the left and digits of
precision after the decimal point.

<p>

With Tcl <code>format</code>, the first argument is a pattern for how
you'd like the final output to look.  Inside the pattern are
placeholders for values.  The second through Nth arguments to
<code>format</code> are the values themselves:

<blockquote>
<code><pre>
format pattern <i>value1 value2 value3 .. valueN</i>
</pre></code>
</blockquote>


We can never figure out how to use format without either copying an
earlier fragment of pattern or referring to the man page 
(<a href="http://www.scriptics.com/man/tcl7.5/TclCmd/format.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/format.n.html</a>).  However, here are some
examples for you to copy:

<blockquote>
<code><pre>
% # format prices with two digits after the point
% format "Price:  %0.2f" 17
Price:  17.00
% # pad some stuff out to fill 20 spaces
% format "%20s" "a long thing"
        a long thing
% format "%20s" "23"
                  23
% # notice that the 20 spaces is a MINIMUM; use string range
% # if you might need to truncate
% format "%20s" "something way longer than 20 spaces"
something way longer than 20 spaces
% # turn a number into an ASCII character
% format "%c" 65
A
</pre></code>
</blockquote>


The Tcl command <code>scan</code> performs the reverse operation,
i.e., parses an input string according to a pattern and stuffs values
as it finds them into variables:

<blockquote>
<code><pre>
% # turn an ASCII character into a number
% scan "A" "%c" the_ascii_value
1
% set the_ascii_value
65
% 
</pre></code>
</blockquote>


Notice that the number returned by <code>scan</code> is a count of how
many conversions it was able to perform successfully.  If you really
want to use <code>scan</code>, you'll need to visit 
the man page: <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/scan.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/scan.n.html</a>.  For an idea of
how useful this is for Web development, consider that the
entire 250,000-line ArsDigita Community System does not contain a
single use of the <code>scan</code> command.


<h3>Reference: String operations</h3>

<UL>

<h4>Commands that don't start with <code>string</code></h4>

<li><code>append <i>variable_name value1 value2 value3 ... valueN</i></code>
<br>
sets the variable defined by variable_name to the concatenation of 
the old value and all the remaining arguments
(<a href="http://www.scriptics.com/man/tcl7.5/TclCmd/append.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/append.n.html</a>)

<li><code>regexp <i>?switches? expression string ?matchVar? ?subMatchVar subMatchVar ...?</i></code>
<br>

Returns 1 if <code>expression</code> matches <code>string</code>; 0
otherwise.  If successful, <code>regexp</code> sets the match
variables to the parts of <code>string</code> that matches the
corresponding parts of <code>expression</code>.

<blockquote>
<code><pre>
% set fraction "5/6"
5/6
% regexp {(.*)/(.*)} $fraction match num denom
1
% set match
5/6
% set num
5
% set denom
6
</pre></code>
</blockquote>

(more:  the pattern matching chapter and <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/regexp.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/regexp.n.html</a>)

<li><code>regsub <i>?switches? expression string substitution_spec result_variable_name</i></code>

<br>

Returns a count of the number of matching items that were found and
replaced.  Primarily called for its effect in setting <code>result_variable_name</code>.  

<p>

Here's an example where we ask a user to type in keywords, separated
by commands.  We then expect to feed this list to a full-text search
indexer that will throw an error if handed two commas in a row.  We
use <code>regsub</code> to clean up what the user typed:

<blockquote>
<code><pre>
# here we destructively modify the variable $query_string'
# replacing every occurrence of one or more commas with a single
# command 
% set query_string "samoyed,, sledding, harness"
samoyed,, sledding, harness
% regsub -all {,+} $query_string "," query_string
2
% set query_string
samoyed, sledding, harness
</pre></code>
</blockquote>


(more:  the pattern matching chapter and <a href="http://www.scriptics.com/man/tcl7.5/TclCmd/regexp.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/regexp.n.html</a>)

<P>

<code>regexp</code> and <code>regsub</code> were dramatically improved
with the Tcl 8.1 release.  For a Web developer the most important
feature is the inclusion of non-greedy regular expressions.  This makes it
easy to match the contents of HTML tags.  See <a
href="http://www.scriptics.com/services/support/howto/regexp81.html">http://www.scriptics.com/services/support/howto/regexp81.html</a>
for a full discussion of the differences.


<h4>Commands that start with <code>string</code></h4>

(all of which are documented at 
<a href="http://www.scriptics.com/man/tcl7.5/TclCmd/string.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/string.n.html</a>)

<br>

<LI>
<TT>string compare <I>string1 string2</I></TT><BR><I> returns 0 if the
two strings are equal, -1
if string1 sorts lexicographically before string2, 1 if
string2 sorts lexicographically before string1:</I>

<blockquote>
<code><pre>
string compare apple applesauce  <i>==> -1</i>
string compare apple Apple <i>==> 1</i>
</pre></code>
</blockquote>


<LI>
<TT>string first <I>string1 string2</I></TT><BR>
<I>returns -1 if string1 is not within string2, else returns the index of the first occurrence.  Indices start
from zero, e.g.,</I> 
<blockquote>
<code><pre>
string first tcl catclaw  <i>==> 2</i>
</pre></code>
</blockquote>


<LI>
<TT>string last <I>string1 string2</I></TT><BR><I> -1 if string1 is not
within string2, else index of last occurrence.</I>

<blockquote>
<code><pre>
string last abra abracadabra <i>==> 7</i>
</pre></code>
</blockquote>


<LI>
<TT>string match <I>pattern string</I></TT><BR><I><I> </I>1 if string
matches pattern, 0 if not.  See the chapter on <a
href="pattern-matching.adp"> pattern matching</a> for an explanation of
patterns.</I>
<blockquote>
<code><pre>
</pre></code>
</blockquote>


<LI>
<TT>string range <I>string i j</I></TT><BR><I> range of characters in string
from index i to j, inclusive.</I>

<blockquote>
<code><pre>
string range catclaw 2 4 <i>==> tcl</i>
</pre></code>
</blockquote>

</LI>

<LI>
<TT>string tolower <I>string</I></TT><BR><I> string in lower case.</I>

<blockquote>
<code><pre>
string compare weBmaster Webmaster <i>=> 1</i>

string compare [string tolower weBmaster] \
               [string tolower Webmaster] <i>=> 0</i>
</pre></code>
</blockquote>

</LI>

<LI>
<TT>string toupper <I>string</I></TT><BR><I> string in upper case.</I>
<blockquote>
<code><pre>
set password "ferrari"
string compare "FERRARI" [string toupper $password] <i>==> 0</i>
</pre></code>
</blockquote>



<LI>

<TT>string trim <I>string ?chars?</I></TT><BR><I> trims <TT>chars</TT>
from right and left side of string; defaults to whitespace.</I>
<blockquote>
<code><pre>
set password [string trim $form_password] ; # see above example
</pre></code>
</blockquote>

</LI>

<LI>
<TT>string trimleft <I>string ?chars?</I></TT><BR><I> trims chars from left
of string; defaults to whitespace.</I>
<blockquote>
<code><pre>
set password [string trimleft $form_password] 
</pre></code>
</blockquote>

</LI>


<LI>
<TT>string trimright <I>string ?chars?</I></TT><BR><I> trims chars from
right
of string; defaults to whitespace.</I><blockquote>
<code><pre>
set password [string trimright $form_password] 
</pre></code>
</blockquote>

</LI>

<LI>
<TT>string wordend <I>string index</I></TT><BR>
<I> index of the first character after the last character of the word
containing index.</I>
<blockquote>
<code><pre>
string wordend "tcl is the greatest" 0 <i>==>3</i>
</pre></code>
</blockquote>

</LI>

<LI>
<TT>string wordstart <I>string index</I></TT><BR><I> index of the first char
of the word containing index.</I>
<blockquote>
<code><pre> 
string wordstart "tcl is the greatest" 5 <i>==> 4</i>
</pre></code>
</blockquote>

</UL>



<p>

<br>
<br>

Continue on to <A HREF="list-operations.adp"> list operations</A>.





<hr>
Return to <a href="index.adp">Table of Contents</a>
<br>
<br>
<address><a href="mailto:lsandon@alum.mit.edu">lsandon@alum.mit.edu</a></address>

<center><a href="/comments/add?page_id=4724">Add a comment</a> | <a href="/links/add?page_id=4724">Add a link</a></center>

</body>
</html>



